@@ -9,7 +9,7 @@ from copy import deepcopy
 import torch
 import torch.nn as nn
 from torch.optim import Adam
-from tqdm import tqdm
+from tqdm.auto import tqdm
 
 from .solvers_utils import PretrainedSolver
 from .networks import FCNN
@@ -571,6 +571,47 @@ class BaseSolver(ABC, PretrainedSolver):
         """
         return 0.0
 
+    def get_residuals(self, *coords, to_numpy=False, best=True, no_reshape=False):
+        r"""Get the residuals of the differential equation (or system of differential equations)
+        evaluated at given points.
+
+        :param coords:
+            The coordinate values where the residual(s) shall be evaluated.
+            If numpy arrays are passed, the method implicitly creates torch tensors with corresponding values.
+        :type coords: tuple[torch.Tensor] or tuple[np.ndarray]
+        :param to_numpy: Whether to return numpy arrays. Defaults to False.
+        :type to_numpy: bool
+        :param best:
+            If set to False, the network from the most recent epoch will be used to evaluate the residuals.
+            If set to True, the network from the epoch with the lowest validation loss will be used to evaluate the
+            residuals. Defaults to True.
+        :type best: bool
+        :param no_reshape: If set to True, no reshaping will be performed on output. Defaults to False.
+        :type no_reshape: bool
+        :return:
+            The residuals evaluated at given points.
+            If there is only one equation in the differential equation system,
+            a single torch tensor (or numpy array) will be returned.
+            If there are multiple equations, a list of torch tensors (or numpy arrays) will be returned.
+            The returned shape will be the same as the first input coordinate, unless `no_reshape` is set to True.
+            Note that the return value will always be torch tensors (even if ``coords`` are numpy arrays)
+            unless `to_numpy` is explicitly set to True.
+        :rtype: list[`torch.Tensor` or `numpy.array`] or `torch.Tensor` or `numpy.array`
+        """
+        coords = [c if isinstance(c, torch.Tensor) else torch.tensor(c) for c in coords]
+        original_shape = coords[0].shape
+        coords = [c.reshape(-1, 1).requires_grad_() for c in coords]
+        solution = self.get_solution(copy=False, best=best)
+        funcs = solution(*coords, to_numpy=False, no_reshape=no_reshape)
+        if isinstance(funcs, torch.Tensor):
+            funcs = [funcs]
+        residuals = self.diff_eqs(*funcs, *coords)
+        if not no_reshape:
+            residuals = [r.reshape(*original_shape) for r in residuals]
+        if to_numpy:
+            residuals = [r.detach().cpu().numpy() for r in residuals]
+        return residuals if len(residuals) > 1 else residuals[0]
+
 
 class BaseSolution(ABC):
     r"""A solution to a PDE/ODE (system).
@@ -589,6 +630,9 @@ class BaseSolution(ABC):
     """
 
     def __init__(self, nets, conditions):
+        if nets is None:
+            raise RuntimeError("The nets cannot be None, check if you disabled validation "
+                               "and used `best`=True with `get_solution` / `get_residual`")
         if isinstance(nets, nn.Module):
             # This is for backward compatibility with the `single_net` option
             # The same torch.nn.Module instance is repeated to form a list of the same length as `conditions`
@@ -602,7 +646,7 @@ class BaseSolution(ABC):
         pass  # pragma: no cover
 
     @deprecated_alias(as_type='to_numpy')
-    def __call__(self, *coords, to_numpy=False):
+    def __call__(self, *coords, to_numpy=False, no_reshape=False):
         r"""Evaluate the solution at certain points.
 
         :param coords: tuple of coordinate tensors, each of shape (n_samples, 1)
@@ -611,9 +655,11 @@ class BaseSolution(ABC):
             If set to True, the call returns a ``numpy.ndarray`` instead of ``torch.Tensor``.
             Defaults to False.
         :type to_numpy: bool
+        :param no_reshape: If set to True, no reshaping will be performed on output. Defaults to False.
+        :type no_reshape: bool
         :return:
             Dependent variables evaluated at given points.
-            The shape of output will be that of the first input coordinate.
+            The shape of output will be that of the first input coordinate, unless `no_reshape` is set to True.
         :rtype: list[`torch.Tensor` or `numpy.array`] or `torch.Tensor` or `numpy.array`
         """
         coords = [c if isinstance(c, torch.Tensor) else torch.tensor(c) for c in coords]
@@ -630,9 +676,11 @@ class BaseSolution(ABC):
                 raise ValueError(f"Unrecognized `as_type` option: '{to_numpy}'")
 
         us = [
-            self._compute_u(net, con, *coords).reshape(original_shape)
+            self._compute_u(net, con, *coords)
             for con, net in zip(self.conditions, self.nets)
         ]
+        if not no_reshape:
+            us = [u.reshape(*original_shape) for u in us]
         if to_numpy:
             us = [u.detach().cpu().numpy() for u in us]
 
